#include "RungeKutta.h"

RungeKutta::RungeKutta(int NumVar, double StepSize){
   dym = new vect_f[NumVar];
   dyt = new vect_f[NumVar];
   dydt = new vect_f[NumVar];
   yt = new vect_f[NumVar];
   n = NumVar;
   no2 = n/2;
   h = StepSize;
   hh=h*0.5;
   h6=h/6.0;
   hsq=h*h;
   hsq02 = hsq*0.5;
}

void RungeKutta::DumbUpdate(double t, vect_f * y, vect_f * yout){

   rkable->derivs(t,y,dydt);
   for (int i=0;i< n;i++)
      yout[i]=y[i]+h*dydt[i];
}

/*
 * y(0,1,...,no2-1,no2,...,n-1)
 *

void RungeKutta::Verlet(double t, vect_f * y, vect_f * yout){

   vect_f v = y+no2;

   rkable->derivs(t,y,dydt);

   for (i=0;i < no2;i++) yt[i]=y[i]+h*v[i]+hsq02*dydt[i+no2];//r(t+dt) = r(t)+dt*v(t)+(1/2)*dt^2*a(t)
   for (i=no2;i < n;i++) dym[i]=y[i]+hh*dydt[i];//v(t+dt/2) = v(t) + (dt/2)*a(t)

   rkable->derivs(t,y,dym);
}
   */







void RungeKutta::rkpp(double t, vect_f * y, vect_f * yout){
   int i;

   double  th=t+hh;

   rkable->derivs(t,y,dydt);
   for (i=0;i < n;i++) yt[i]=y[i]+hh*dydt[i];
   rkable->derivs(th,yt,dyt);
   for (i=0;i< n;i++) yt[i]=y[i]+hh*dyt[i];
   rkable->derivs(th,yt,dym);
   for (i=0;i< n;i++) {
      yt[i]=y[i]+h*dym[i];
      dym[i] += dyt[i];
   }
   rkable->derivs(t+h,yt,dyt);
   for (i=0;i< n;i++)
      yout[i]=y[i]+h6*(dydt[i]+dyt[i]+2.0*dym[i]);
}


double RungeKutta::getStepSize() {
   return h;
}

