
import numpy
from scipy import *
from pylab import *
import time
from scipy import weave
from scipy.weave import converters

class LongRangeSpinGlass:
    def __init__(self,Beta=1.0,J_coupling=None,spins=None,size=32):
        self.Beta=Beta
        self.n = size
        if spins is None:
            self.s = ones((size),dtype=int)
        else:
            self.s = spins
        if J_coupling is None:
            self.J = standard_normal((size,size))
        else:
            self.J= J_coupling
        #allows you to play around with different variants of the monte carlo program:
        self.iterate = self.iterate_scipy
        
    def iterate_weave(self):
       n = self.n
       r = random((n))
       s = self.s
       J = self.J
       beta = float(self.Beta)

       code = """
           #line 30 "SpinGlass.py"
           
       for(int x=0; x < n; x++) {
           float field  = 0;
           for(int y=0; y < n; y++) {
               field +=  J(x,y)*s(y);
           }

           float delta_e = s(x)*field;
           if (exp(-beta*delta_e) > r(x)) {
               s(x) = -s(x);
           }
       }
               """
       weave.inline(code, ['J', 's', 'r', 'beta', 'n'], type_converters=converters.blitz, compiler = 'gcc')
       return


    def iterate_kludge_scipy(self):
       n = self.n
       r = random((n))
       s = self.s
       J = self.J
       beta = self.Beta

       delta_e = s*dot(J,s)
       s = (2*(signbit(exp(-beta*delta_e) - r))-1) * s
       return

    def iterate_scipy(self):
       n = self.n
       r = random((n))
       s = self.s
       J = self.J
       beta = self.Beta
       for x in range(n):
            field = dot(J[x,:],s[:])
            delta_e = s[x]*field
            if exp(-beta*delta_e) > r[x]:
                s[x] = -s[x]
       return



    def move(self,num_its):
        for i in range(num_its):
            self.iterate()
        return

    def Energy(self):
       s = self.s
       J = self.J
       n = self.n


       code = """
           #line 56 "ParTempSpinGlass.py"
           int x,y;
           int mod = n-1;
           float e_tot = 0;
           
           for(x=0; x < n; x++) {
             for(y=0; y < n; y++) {
                e_tot -= s(x)*(J(x,y)*s(y);
                    }
                 }
            return_val=e_tot*0.5;
               """
       energy = weave.inline(code, ['J', 's', 'n'], type_converters=converters.blitz, compiler = 'gcc')
       return energy

def encode_mult(seq):
    n = seq.size
    out = zeros((n),dtype= int)
    out[0] = seq[0]
    for i in range(1,n): 
        out[i] = out[i-1] ^ seq[i]
    return out

def decode_mult(seq):
    n = seq.size
    out = zeros((n),dtype= int)
    out[n-1] = seq[n-1]
    for i in range(n-2,-1,-1): 
        out[i] = seq[i+1] ^ seq[i]
    return out

def encode_dummy(seq):
    return seq

def decode_dummy(seq):
    return seq


import os
import re

num_images=0
J=array([[]])
n=0
encode = encode_dummy
decode = decode_dummy


for rootdir, dirs, files in os.walk('images/'):
    for file in files:
        if re.search(".png",file):
            colorimage = imread(os.path.join(rootdir, file))
            num_images += 1
            image = colorimage[:,:,0]
            image = image.astype(int)
            if n == 0:
                n= image.size
                J = zeros((n,n))
            else:
                if n != image.size:
                    print "error, size of image should be n = ", n, "not ", image.size
            image = reshape(image,(n))
            image = encode(image)
                
            for i in range(n):
                for j in range(n):
		    if i != j:
                       J[i,j] += (2*image[i]-1)*(2*image[j]-1)

J = J/n

    
#colorimage = imread("images/face.png")
colorimage = imread("images/square.png")
#colorimage = imread("half_face.png")

image = colorimage[:,:,0]
image = image.astype(int)
image = reshape(image,(n))
image = encode(image)
s = 2*image-1

sg = LongRangeSpinGlass(Beta = 1/0.1, spins = s, J_coupling=J, size=n)

sg.move(100)

#to play around with annealing the spins, you can do:
#sg.Beta = 1/0.2
#sg.move(100)
sg.Beta = 1/0.05
sg.move(100)

sg.s = (1+sg.s)/2
sg.s = decode(sg.s)
sg.s = reshape(sg.s,(16,16))
colormap = cm.gist_gray
imshow(sg.s,interpolation="nearest",cmap=colormap)
show()



